@@ -1,6 +1,7 @@
 import bisect
 import collections
 import errno
+import operator
 import sys
 import time
 import weakref
@@ -9,12 +10,13 @@ from greenhouse import compat
 
 
 __all__ = ["pause", "pause_until", "pause_for", "schedule", "schedule_at",
-        "schedule_in", "schedule_recurring", "add_exception_handler", "greenlet"]
+        "schedule_in", "schedule_recurring", "schedule_exception",
+        "schedule_exception_at", "schedule_exception_in", "end",
+        "add_exception_handler", "greenlet", "handle_exception"]
 
 _exception_handlers = []
 
-FAST_POLL_TIMEOUT = 0.001
-SLOW_POLL_TIMEOUT = 1.0
+POLL_TIMEOUT = 1.0
 
 
 state = type('GreenhouseState', (), {})()
@@ -34,6 +36,9 @@ state.descriptormap = collections.defaultdict(list)
 # lined up to run right away
 state.to_run = collections.deque()
 
+# exceptions queued up for scheduled coros
+state.to_raise = weakref.WeakKeyDictionary()
+
 
 def _hit_poller(timeout, interruption=None):
     interruption = interruption or (lambda: False)
@@ -64,7 +69,7 @@ def _hit_poller(timeout, interruption=None):
 
         map(objs.pop, removals[::-1])
         if not objs:
-            state.descriptormap.pop(fd)
+            state.descriptormap.pop(fd, None)
 
         if eventmap & (state.poller.INMASK | state.poller.ERRMASK):
             for sock in socks:
@@ -192,6 +197,44 @@ def schedule_recurring(interval, target=None, maxtimes=0, starting_at=0,
 
     return target
 
+def schedule_exception(exception, target):
+    '''set a greenlet to have *exception* raised in it
+
+    *target* must be a greenlet, so unlike schedule(), this can not be a
+    decorator'''
+    if not isinstance(target, compat.greenlet):
+        raise TypeError("can only schedule exceptions for greenlets")
+    if target.dead:
+        raise ValueError("can't send exceptions to a dead greenlet")
+    schedule(target)
+    state.to_raise[target] = exception
+
+def schedule_exception_at(unixtime, exception, target):
+    '''set a greenlet to have *exception* raised in it at a timestamp
+
+    *target* must be a greenlet. *exception* will be raised in it sometime
+    after *unixtime*, a timestamp'''
+    if not isinstance(target, compat.greenlet):
+        raise TypeError("can only schedule exceptions for greenlets")
+    if target.dead:
+        raise ValueError("can't send exceptions to a dead greenlet")
+    schedule_at(unixtime, target)
+    state.to_raise[target] = exception
+
+def schedule_exception_in(secs, exception, target):
+    '''set a greenlet have *exception* raised in it *secs* seconds later
+
+    *target* must be a greenlet. *exception* will be raised in it sometime
+    after *secs* seconds have elapsed'''
+    schedule_exception_at(time.time() + secs, exception, target)
+
+def end(target):
+    '''schedule a greenlet to be killed abruptly
+
+    *target* must be a greenlet. it will immediately be scheduled with a
+    compat.GreenletExit to be raised in it'''
+    schedule_exception(compat.GreenletExit(), target)
+
 def schedule_to_top(target=None, args=(), kwargs=None):
     '''set up a function or greenlet to run, skipping to the front of the line
 
@@ -227,26 +270,32 @@ def mainloop():
             break
         try:
             if not state.to_run:
-                _hit_poller(FAST_POLL_TIMEOUT)
+                _hit_poller(0)
                 _check_paused()
 
                 while not state.to_run:
                     # if there are timed-paused greenlets, we can
                     # just wait until the first of them wakes up
                     if state.timed_paused:
-                        until = state.timed_paused[0][0] + FAST_POLL_TIMEOUT
+                        until = state.timed_paused[0][0] + 0.001
                         _hit_poller(until - time.time(), _interruption_check)
                         _check_paused(True)
                     else:
-                        _hit_poller(SLOW_POLL_TIMEOUT, _interruption_check)
+                        _hit_poller(POLL_TIMEOUT, _interruption_check)
 
-            state.to_run.popleft().switch()
+            target = state.to_run.popleft()
+            exc = state.to_raise.pop(target, None)
+            if exc is not None:
+                target.throw(exc)
+            else:
+                target.switch()
         except Exception, exc:
             if sys:
-                _consume_exception(*sys.exc_info())
+                handle_exception(*sys.exc_info())
 state.mainloop = mainloop
 
-def _consume_exception(klass, exc, tb):
+def handle_exception(klass, exc, tb):
+    "run all registered exception handlers"
     _purge_exception_handlers()
 
     for weak in _exception_handlers:
@@ -258,9 +307,8 @@ def _consume_exception(klass, exc, tb):
             pass
 
 def _purge_exception_handlers():
-    bad = [i for i, weak in enumerate(_exception_handlers) if weak() is None]
-    for i in bad[::-1]:
-        _exception_handlers.pop(i)
+    globals()['_exception_handlers'] = [weak for weak in _exception_handlers
+            if weak()]
 
 def add_exception_handler(handler):
     if not hasattr(handler, "__call__"):
